/*
 * Copyright 2021 VIA Technologies, Inc. All Rights Reserved.
 *
 * This PROPRIETARY SOFTWARE is the property of WonderMedia Technologies, Inc.
 * and may contain trade secrets and/or other confidential information of
 * WonderMedia Technologies, Inc. This file shall not be disclosed to any third
 * party, in whole or in part, without prior written consent of WonderMedia.
 *
 * THIS PROPRIETARY SOFTWARE AND ANY RELATED DOCUMENTATION ARE PROVIDED AS IS,
 * WITH ALL FAULTS, AND WITHOUT WARRANTY OF ANY KIND EITHER EXPRESS OR IMPLIED,
 * AND WonderMedia TECHNOLOGIES, INC. DISCLAIMS ALL EXPRESS OR IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
*/
#include <WiFi.h>
#include <Harpcam.h>
#include <WebSockets2_Generic.h>

using namespace websockets2_generic;

//default connect to A950
const char* ssid = "via-harp";
const char* password = "via-harp";
const char* uuid = "14e3fea6-9d8a-3b09-a2f5-d686d2942266";

WebsocketsClient client;
Harpcam harpcam;
const int ledPin = 4;

void onEventsCallback(WebsocketsEvent event, String data) 
{
  (void) data;
  
  if (event == WebsocketsEvent::ConnectionOpened) 
  {
    Serial.println("Connnection Opened");
  } 
  else if (event == WebsocketsEvent::ConnectionClosed) 
  {
    Serial.println("Connnection Closed");
  } 
  else if (event == WebsocketsEvent::GotPing) 
  {
    Serial.println("Got a Ping!");
  } 
  else if (event == WebsocketsEvent::GotPong) 
  {
    Serial.println("Got a Pong!");
  }
}

void onMessagesCallback(WebsocketsMessage message)
{
  String msg_data = message.data();

  if (harpcam.parseJsonContent(msg_data)) {
    if (harpcam.isDetected()) {
      //digitalWrite(ledPin, HIGH);
      int num = harpcam.getNumOfObjects();
      Serial.printf("getNumOfObjects: %d\n", num);
      Harpcam::OutputData output[num];
      harpcam.getOutputData(output);

      for (int i=0; i<num; i++) {
        Serial.printf("num: %d\n", i);
        Serial.printf("type:%d, x:%d, y:%d, w:%d, h:%d, c:%f\n", 
          output[i].type, output[i].x, output[i].y, output[i].w, output[i].h, output[i].confidence);
      }
    }
    //else
    //  digitalWrite(ledPin, LOW);

    client.send(harpcam.process());
  }
}

void setup() {
  Serial.begin(115200);
  while (!Serial);

  // initialize digital pin ledPin as an output
  pinMode(ledPin, OUTPUT);

  for(uint8_t t = 4; t > 0; t--) {
    Serial.printf("[SETUP] BOOT WAIT %d...\n", t);
    Serial.flush();
    delay(1000);
  }
  Serial.println("\nStarting Harpcam example on " + String(ARDUINO_BOARD));

  if (!harpcam.setupCamera())
    return;

  // set UUID
  harpcam.setUUID(uuid);

  Harpcam::EFunc func_id = Harpcam::FUNC_NEURAL_NETWORK;
  harpcam.setFuncID(func_id);

  // Connect to wifi
  WiFi.begin(ssid, password);

  // Wait some time to connect to wifi
  for (int i = 0; i < 10 && WiFi.status() != WL_CONNECTED; i++) {
    Serial.print(".");
    delay(1000);
  }

  // Check if connected to wifi
  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("No Wifi!");
    return;
  }

  // run callback when messages are received
  client.onMessage(onMessagesCallback);

  // run callback when events are occuring
  client.onEvent(onEventsCallback);

  // try to connect to Websockets server
  bool connected = client.connect(harpcam.host, harpcam.port, harpcam.target);

  if (connected) {
    client.send(harpcam.begin());
  }
}

void loop() {
  if (client.available()) {
    client.poll();
  }
}
